#ifndef KYSEC_MEMDB_H__
#define KYSEC_MEMDB_H__

#include <stddef.h>

//存储字段数据类型
// #define KYSEC_MEMDB_NULL            1
// #define KYSEC_MEMDB_RECORD          2
#define KYSEC_MEMDB_INT             3
#define KYSEC_MEMDB_DOUBLE          4
#define KYSEC_MEMDB_STR             5
#define KYSEC_MEMDB_UNDEFINE        99

//查询条件类型
#define KYSEC_MEMDB_COND_EQUAL       0x0001      /** = */
#define KYSEC_MEMDB_COND_NOT_EQUAL   0x0002      /** != */
#define KYSEC_MEMDB_COND_LESSTHAN    0x0004      /** < */
#define KYSEC_MEMDB_COND_GREATER     0x0008      /** > */
#define KYSEC_MEMDB_COND_LTEQUAL     0x0010      /** <= */
#define KYSEC_MEMDB_COND_GTEQUAL     0x0020      /** >= */

//kysec内存数据库记录最大字段数量
#define MAX_SECTION_LEN 16

#define KYSEC_MEMDB_OK                          0
#define KYSEC_MEMDB_ERROR_GENERIC               -1000
#define KYSEC_MEMDB_ERROR_INTERNAL              -1001
#define KYSEC_MEMDB_ERROR_BUSY                  -1002

//kysec内存数据库基础数据
typedef struct
{
    int type;           //数据类型， 3 int， 4 double， 5 string
    void *dptr;         //数据指针，指向数据存储地址
}kysec_memdb_value;

//kysec内存数据库数据记录
typedef struct
{
    int length;                                 //记录字段数量
    kysec_memdb_value value[MAX_SECTION_LEN];    //记录字段数据
}kysec_memdb_record;

//
typedef struct
{
    int cond;
    int column;
    kysec_memdb_value value;
}kysec_memdb_query_cond;

typedef struct
{
    int length;
    kysec_memdb_query_cond list[MAX_SECTION_LEN];
}kysec_memdb_query_args;

typedef int (*memdb_parse)(kysec_memdb_record *record, void *private_data);

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 
 * 
 * @param id 
 * @param size 
 * @return void * 
 */
void *kysec_memdb_create_database(int id, size_t size);

/**
 * @brief 
 * 
 * @param id 
 * @return int 
 */
int kysec_memdb_delete_database(int id);

/**
 * @brief 
 * 
 * @param id 
 * @param handle 
 * @return int 
 */
void *kysec_memdb_connect_database(int id);

/**
 * @brief 
 * 
 * @param handle 
 * @return int 
 */
int kysec_memdb_disconnect_database(void *handle);

/**
 * @brief 
 * 
 * @param handle 
 * @param col 
 * @return int 
 */
int kysec_memdb_create_t_tree_index(void *handle, int col);

/**
 * @brief 
 * 
 * @param handle 
 * @param col 
 * @return int 
 */
int kysec_memdb_drop_t_tree_index(void *handle, int col);

/**
 * @brief 
 * 
 * @param handle 
 * @param col 
 * @return int 
 */
int kysec_memdb_check_t_tree_index(void *handle, int col);

/**
 * @brief 
 * 
 * @param handle 
 * @param list 
 * @param size 
 * @return int 
 */
int kysec_memdb_create_record(void *handle, kysec_memdb_record record);

/**
 * @brief 
 * 
 * @param handle 
 * @param rec 
 * @return int 
 */
int kysec_memdb_delete_record(void *handle, kysec_memdb_record record);

/**
 * @brief 
 * 
 * @param handle 
 * @param args 
 * @param private_data 
 * @return int 
 */
int kysec_memdb_retrive(void *handle, kysec_memdb_query_args args, memdb_parse func, void *private_data);

/**
 * @brief 
 * 
 * @param handle 
 * @return long 
 */
long kysec_memdb_size(void *handle);

/**
 * @brief 
 * 
 * @param handle 
 * @return long 
 */
long kysec_memdb_freesize(void *handle);


/*=====================================advance functions=====================================*/
/**
 * @brief
 *
 * @param handle
 * @param query
 * @return int
 */
void *kysec_memdb_begin_query(void *handle, kysec_memdb_query_args args);

/**
 * @brief
 *
 * @param handle
 * @param rec
 * @return int
 */
void *kysec_memdb_fetch(void *handle, void *query);

/**
 * @brief
 *
 * @param handle
 * @param query
 * @return int
 */
void kysec_memdb_finish_query(void *handle, void *query);

/**
 * @brief 
 * 
 * @param handle 
 * @param spec_rec 
 * @return int 
 */
int kysec_memdb_delete_specific_record(void *handle, void *spec_rec);

/**
 * @brief 
 * 
 * @param handle 
 * @param spec_rec 
 * @param record 
 * @return int 
 */
int kysec_memdb_update_specific_record(void *handle, void *spec_rec, kysec_memdb_record record);

/*=====================================advance functions=====================================*/

#ifdef __cplusplus
}
#endif

#endif
